%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% chart_spectrum.sl
%
% Version 1.0.1 (2004/02/18)
%
% Author:  Christopher J. Stawarz (cstawarz@cfa.harvard.edu)
%
% Purpose:
%   Create a source spectrum suitable for input to the Chandra
%   Ray Tracer (ChaRT) from a source model defined in Sherpa.
%
% Thread:
%   http://cxc.harvard.edu/chart/threads/prep/
%
% Functions:
%   get_chart_spectrum()
%     - returns spectrum data in two arrays: energy (keV) and
%       flux (photons/cm**2/s)
%
%   write_chart_spectrum()
%     - writes spectrum to output file, which can then be used
%       as input to ChART
%
%   plot_chart_spectrum()
%     - plots spectrum
%
% Usage Notes:
%   These functions are designed to be used within Sherpa (or,
%   alternatively, within a S-Lang script that has imported the
%   sherpa module).  Before using any of them, the user or script
%   must read in a dataset, establish source and instrument
%   models, and perform a fit to the source model.
%
% Programming Notes:
%   The regridding algorithm in get_chart_spectrum() uses a
%   numerical tolerance when deciding whether two energy values
%   are equal.  By default, the tolerance is 0.0005 keV.  Users
%   can change the tolerance by changing the value of the
%   variable "chart_spectrum->tol".
%   
%   This code assumes that the following functions in the sherpa
%   module return data in the specified units (which should 
%   always be the case for Chandra data):
%
%     get_photon_energy_axes():	keV
%     get_source():		photons/cm**2/s/keV
%     get_arf_axes():		keV
%
% History:
%   2004/02/18 - Cosmetic tweaks: added explicit "static"
%     declaration to static variables and functions; added
%     author's name to comments
%   2003/07/22 - initial version, created for CIAO 3.0
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


implements("chart_spectrum");


%
% Wrapper around S-Lang error() function
%

private define do_error(msg) { error("Error: " + msg); }


%
% Floating-point comparison functions
%
% These are used to compare energy grid values.  They replace
% the operators ==, <, <=, >, >= and take into account the
% condition that if (abs(x-y) <= tol) is true, then x and y
% are considered equal.
%

static variable tol = 0.0005;	% Tolerance

% x == y
static define tol_eq(x,y) { return (abs(x-y) <= tol); }

% x < y
static define tol_lt(x,y) {
    return (andelse { not tol_eq(x,y) } { x < y });
}

% x <= y
static define tol_lteq(x,y) {
    return (orelse { tol_eq(x,y) } { x < y });
}

% x > y
static define tol_gt(x,y) {
    return (andelse { not tol_eq(x,y) } { x > y });
}

% x >= y
static define tol_gteq(x,y) {
    return (orelse { tol_eq(x,y) } { x > y });
}


%
% get_chart_spectrum()
%
% Usage:
%   (energy,flux) = get_chart_spectrum();
%   (energy,flux) = get_chart_spectrum(dataset);
%   (energy,flux) = get_chart_spectrum(energy_min,enery_max);
%   (energy,flux) = get_chart_spectrum(dataset,energy_min,enery_max);
%
% Purpose:
%   Given the dataset number of a Sherpa source model, creates
%   a spectrum from that source model in units appropriate for
%   input to ChaRT (energy in keV, flux in photons/cm**2/s).
%
% Usage Notes:
%   The "dataset" parameter must be of type Integer_Type and
%   indicates the Sherpa dataset for which to generate a
%   spectrum.  If the parameter is not supplied, dataset 1 is
%   used.
%
%   The "energy_min" and "energy_max" parameters must be numeric
%   and indicate the minimum and maximum energy values, respectively,
%   in the output spectrum.  If either value is NULL, the
%   corresponding default boundary (as determined by Sherpa) will be
%   used.
%
%   The return values are two arrays containing the spectrum data
%   (energy and flux).
%
% Programming Notes:
%   This function takes the data returned by
%   get_photon_energy_axes() and get_source(), converts the
%   flux values from photons/cm**2/s/keV to photons/cm**2/s, and
%   then regrids the spectrum to match the ARF's energy grid.
%   The regridding is necessary because if the ARF and RMF are
%   on different grids, the bins returned by
%   get_photon_energy_axes() may be of variable width.
%   Regridding the spectrum to match the ARF grid ensures that
%   the bin widths are uniform.
%

public define get_chart_spectrum() {
    variable dataset = NULL;	% Dataset number
    variable energy_min = NULL;	% Lower energy bound for spectrum
    variable energy_max = NULL;	% Upper energy bound for spectrum

    %
    % Store and validate input parameters
    %

    switch (_NARGS)
      { case 0 : ; }
      { case 1 : dataset = (); }
      { case 2 : (energy_min,energy_max) = (); }
      { case 3 : (dataset,energy_min,energy_max) = (); }
      { usage(_function_name() +
          "([dataset][,][energy_min,energy_max])");
      }

    if (dataset == NULL) {
	dataset = 1;
    } else if (typeof(dataset) != Integer_Type) {
        do_error("dataset number must be an integer");
    }

    if (energy_min != NULL) {
        energy_min = typecast(energy_min,Double_Type);
    }
    if (energy_max != NULL) {
        energy_max = typecast(energy_max,Double_Type);
    }


    %
    % Get input spectral data
    %

    % Get energy axis

    variable pe_axis = get_photon_energy_axes(dataset);

    if (pe_axis == NULL) {
	do_error("unable to retrieve energy grid for dataset "
	  + string(dataset));
    } else if ((pe_axis.lo == NULL) or (pe_axis.hi == NULL)) {
	do_error("energy grid for dataset " + string(dataset) +
	  " contains no bin-width information");
    }

    % Get source amplitudes

    variable source = get_source(dataset);

    if (source == NULL) {
	do_error("unable to retrieve source model photon " +
	  "amplitudes for dataset " + string(dataset));
    }

    % Calculate flux values on the input grid in photons/cm**2/s
    variable in_flux = (pe_axis.hi - pe_axis.lo) * source;


    %
    % Initialize output spectral data
    %

    % Get ARF energy axis

    variable arf_axis = get_arf_axes(dataset);

    if (arf_axis == NULL) {
	do_error("unable to retrieve ARF energy grid for dataset " +
	  string(dataset));
    } else if ((arf_axis.lo == NULL) or (arf_axis.hi == NULL)) {
	do_error("ARF energy grid for dataset " + string(dataset) +
	  " contains no bin-width information");
    }

    % Calculate output energy array
    variable out_energy = (arf_axis.lo + arf_axis.hi)
      / (typecast(2,_typeof(arf_axis.lo)));

    % Ensure that each arf_axis bin lies completely within
    % a pe_axis bin (the bin boundaries may coincide)

    variable min_lo = pe_axis.lo[0];
    variable max_hi = pe_axis.hi[length(pe_axis.hi)-1];

    variable arf_first = 0;
    variable arf_last = length(arf_axis.lo) - 1;

    while ( tol_lt(arf_axis.lo[arf_first],min_lo) )  arf_first++;
    while ( tol_gt(arf_axis.hi[arf_last],max_hi) )  arf_last--;

    % Ensure that all arf_axis bin midpoints (i.e. all out_energy
    % values) lie within energy range specified by user

    if (energy_min != NULL) {
        while ( tol_lt(out_energy[arf_first],energy_min) )
            arf_first++;
    }

    if (energy_max != NULL) {
        while ( tol_gt(out_energy[arf_last],energy_max) )
            arf_last--;
    }

    % Resize arrays appropriately

    arf_axis.lo = arf_axis.lo[[arf_first:arf_last]];
    arf_axis.hi = arf_axis.hi[[arf_first:arf_last]];
    out_energy = out_energy[[arf_first:arf_last]];

    % Create empty output flux array
    variable out_flux = (_typeof(in_flux))[length(out_energy)];


    %
    % Calculate output flux on ARF energy grid
    %

    variable flux_val;	% Flux for current arf_axis bin
    variable arf = 0;	% Bin counter for arf_axis bins
    variable pe = 0;	% Bin counter for pe_axis bins

    loop (length(out_flux)) {
	% Step through pe_axis bins until we find one that the
	% current arf_axis bin overlaps
	while ( tol_gteq(arf_axis.lo[arf],pe_axis.hi[pe]) )  pe++;

	% Calculate flux for current arf_axis bin; calculation
	% depends on whether arf_axis bin overlaps only one or
	% multiple pe_axis bins

	if ( tol_lteq(arf_axis.hi[arf],pe_axis.hi[pe]) ) {
	    % Case 1:
	    % The current arf_axis bin is completely contained in
	    % the current pe_axis bin

	    flux_val = arf_axis.hi[arf] - arf_axis.lo[arf];
	    flux_val /= pe_axis.hi[pe] - pe_axis.lo[pe];
	    flux_val *= in_flux[pe];

	    out_flux[arf] = flux_val;
	} else {
	    % Case 2:
	    % The current arf_axis bin overlaps two or more pe_axis
	    % bins

	    % Add flux from first overlapped bin

	    flux_val = pe_axis.hi[pe] - arf_axis.lo[arf];
	    flux_val /= pe_axis.hi[pe] - pe_axis.lo[pe];
	    flux_val *= in_flux[pe];

	    out_flux[arf] = flux_val;
	    pe++;

	    % Add flux from any middle overlapped bins (i.e. pe_axis
	    % bins that are overlapped completely by the current
	    % arf_axis bin)

	    while ( tol_gt(arf_axis.hi[arf],pe_axis.hi[pe]) ) {
		out_flux[arf] += in_flux[pe];
		pe++;
	    }

	    % Add flux from the last overlapped bin

	    flux_val = arf_axis.hi[arf] - pe_axis.lo[pe];
	    flux_val /= pe_axis.hi[pe] - pe_axis.lo[pe];
	    flux_val *= in_flux[pe];

	    out_flux[arf] += flux_val;
	}

	% Move on to the next arf_axis bin
	arf++;
    }

    % Return output energy and flux arrays
    return (out_energy,out_flux);
}


%
% write_chart_spectrum()
%
% Usage:
%   write_chart_spectrum(outfile);
%   write_chart_spectrum(dataset,outfile);
%   write_chart_spectrum(outfile,energy_min,energy_max);
%   write_chart_spectrum(dataset,outfile,energy_min,energy_max);
%
% Purpose:
%   Create a ChaRT input spectrum using get_chart_spectrum()
%   and write it to an ASCII output file.
%
% Usage Notes:
%   The "outfile" parameter must be of type String_Type and
%   specifies the name of the spectrum file to create.  If
%   the file already exists, it will be overwritten.  The
%   other parameters are identical to the corresponding
%   parameters to get_chart_spectrum().
%

public define write_chart_spectrum() {
    variable outfile;		% Name of output file
    variable dataset = NULL;	% Dataset number
    variable energy_min = NULL;	% Lower energy bound for spectrum
    variable energy_max = NULL;	% Upper energy bound for spectrum

    % Store and validate input parameters

    switch (_NARGS)
      { case 1 : outfile = (); }
      { case 2 : (dataset,outfile) = (); }
      { case 3 : (outfile,energy_min,energy_max) = (); }
      { case 4 : (dataset,outfile,energy_min,energy_max) = (); }
      { usage(_function_name() +
          "([dataset,]outfile[,energy_min,energy_max])");
      }

    if (typeof(outfile) != String_Type)
        do_error("output file name must be a string");

    % Get spectrum data

    variable out_energy;	% Output energy array
    variable out_flux;		% Output flux array

    (out_energy,out_flux) = get_chart_spectrum(dataset,
      energy_min,energy_max);

    % Write spectrum
    writeascii(outfile,out_energy,out_flux);
}


%
% plot_chart_spectrum()
%
% Usage:
%   plot_chart_spectrum();
%   plot_chart_spectrum(dataset);
%   plot_chart_spectrum(energy_min,energy_max);
%   plot_chart_spectrum(dataset,energy_min,energy_max);
%
% Purpose:
%   Create a ChaRT input spectrum using get_chart_spectrum()
%   and plot it.
%
% Usage Notes:
%   All parameters are identical to the corresponding parameters
%   to get_chart_spectrum().
%

public define plot_chart_spectrum() {
    variable dataset = NULL;	% Dataset number
    variable energy_min = NULL;	% Lower energy bound for spectrum
    variable energy_max = NULL;	% Upper energy bound for spectrum

    % Store and validate input parameters

    switch (_NARGS)
      { case 0 : ; }
      { case 1 : dataset = (); }
      { case 2 : (energy_min,energy_max) = (); }
      { case 3 : (dataset,energy_min,energy_max) = (); }
      { usage(_function_name() +
          "([dataset][,][energy_min,energy_max])");
      }

    % Get spectrum data

    variable out_energy;	% Output energy array
    variable out_flux;		% Output flux array

    (out_energy,out_flux) = get_chart_spectrum(dataset,
      energy_min,energy_max);

    % Plot spectrum, with appropriate labels

    chips_clear();
    () = curve(out_energy,out_flux);
    () = chips_eval("location 0.2 0.95 0.1 0.9");
    () = chips_eval("xlabel 'Energy (keV)'");
    () = chips_eval("xlabel size 1.5");
    () = chips_eval("ylabel 'Flux (photons/cm^{2}/s)'");
    () = chips_eval("ylabel size 1.5");
    () = chips_eval("title 'ChaRT Source Spectrum'");
    () = chips_eval("title size 1.5");
    () = chips_eval("tickvals size 1.5");
    chips_redraw();
}

