%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% cxchistory.sl
%
% Version 1.0 (2006/02/09)
%
% Author:  Christopher J. Stawarz (cstawarz@cfa.harvard.edu)
%
% Purpose:
%   Provides S-Lang functions for writing program parameter history in the
%   standard Chandra X-ray Center format to the header of a FITS file.
%
% Public Functions:
%   cxchistory_bundle_params()
%   cxchistory_create_history()
%   cxchistory_write_history()
%
% Required Modules:
%   - paramio
%   - varmm or cfitsio (the slight differences between the FITS I/O functions
%     provided by these modules are taken into account)
%
% History:
%   2004/01/22 - initial version (0.1)
%   2006/02/09 - version 1.0: changed all "Fits_File_Type" to
%		 "ciao_Fits_File_Type"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Import required modules (if necessary)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


#ifnexists paramopen
import("paramio");
#endif

#ifnexists _fits_open_file
import("varmm");
#endif


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Define private variables
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Width of "content" portion of history line
private variable HISTWIDTH = 56;

% printf-style format specifier for history line
private variable HISTFMT = " %-4.4s  :%-" + string(HISTWIDTH) +
  "." + string(HISTWIDTH) + "sASC%05d";


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Define private functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


private define create_history_lines()
%
% Given key, histstr, and histnum (see descriptions below), creates history
% lines and pushes them onto the stack.  Also returns the updated value of
% HISTNUM.
%
{
    variable key;		% Key for history line ("TOOL", "PARM", etc.)
    variable histstr;		% Content of history line
    variable histnum;		% Current value of HISTNUM

    (key,histstr,histnum) = ();

    variable len;	% Length of history string
    variable i;		% Current position in string
    variable n;		% Number of characters to extract

    len = strlen(histstr);

    % Create a new history line for each HISTWIDTH-long substring of
    % "histstr"

    for (i=1; i<len; i+=n) {
	% If number of characters remaining is less than HISTWIDTH,
	% extract all remaining characters; otherwise, extract HISTWIDTH
	% characters 
	n = min([len-i+1, HISTWIDTH]);

	% Update histnum
	histnum++;

	% Create new history line and push it on to stack
	sprintf(HISTFMT,key,substr(histstr,i,n),histnum);

	% Set key to "CONT" for all lines after the first
	key = "CONT";
    }

    % Return new value of HISTNUM, plus all items on stack
    % (i.e. all history lines)
    return histnum;
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Define public functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


public define cxchistory_bundle_params()
%
% Takes Param_File_Type object and returns associative array: keys are
% parameter names, values are parameter values.  Output can be used as an
% argument to cxchistory_create_history().
%
{
    variable parfile;		% Parameter file object

    % Usage message
    variable usage_msg = sprintf("Assoc_Type params = %s(" +
      "Param_File_Type parfile)",_function_name());

    switch (_NARGS)
	{ case 1 : parfile = (); }
	{ usage(usage_msg); }

    if (typeof(parfile) != Param_File_Type)  usage(usage_msg);

    variable params = Assoc_Type[String_Type];
    variable name;
    variable value;

    foreach (plist_names(parfile)) {
	name = ();
	value = pget(parfile,name);
	params[name] = value;
    }

    return params;
}


public define cxchistory_create_history()
%
% Given input parameters (see descriptions below), returns array of history
% lines (strings) and value of HISTNUM keyword, suitable for passing to
% cxchistory_write_history().
%
{
    %
    % Store and validate arguments
    %

    variable progname;		% Name of program for which to write history
    variable old_histnum;	% Initial number of history lines in file
    variable paramlist = NULL;	% Associative array of parameter/value pairs
    variable stacklist = NULL;	% Associative array: keys are names of
    				% parameters that support stacks, values are
				% arrays of strings containing stack expansions

    % Usage message

    variable usage_msg = "\n  (histarray,new_histnum) = " + _function_name() +
        "(toolname,old_histnum[,paramlist][,stacklist])\n\n" +
	"Parameters:\n" +
	"  toolname    : name of tool for which to write history\n" +
	"  old_histnum : current value of HISTNUM keyword\n" +
	"  paramlist   : associative array of parameter name/value pairs\n" +
	"  stacklist   : associative array: keys are names of parameters\n" +
	"                that support stacks, values are arrays of strings\n" +
	"                containing stack expansions\n";

    % Get arguments

    switch (_NARGS)
	{ case 2 : (progname,old_histnum) = (); }
	{ case 3 : (progname,old_histnum,paramlist) = (); }
	{ case 4 : (progname,old_histnum,paramlist,stacklist) = (); }
	{ usage(usage_msg); }

    % Validate "progname", "old_histnum", and "paramlist"

    if (orelse
	   % "progname" must be a string
	   { typeof(progname) != String_Type }

	   % "old_histnum" must be an integer
	   { typeof(old_histnum) != Integer_Type }

	   % "paramlist" must be NULL or an associative array (of any type)
	   { andelse
	       { paramlist != NULL }
	       { typeof(paramlist) != Assoc_Type }
	   }
	) {
	% Die if any argument was of incorrect type
        usage(usage_msg);
    }

    % Validate "stacklist" (must be NULL or an associative array of
    % arrays of strings)

    if (stacklist != NULL) {
	% Die if "stacklist" isn't an associate array
	if (typeof(stacklist) != Assoc_Type)  usage(usage_msg);

	variable key;	% Current key in "stacklist"

	foreach (assoc_get_keys(stacklist)) {
	    key = ();

	    % Die if current item isn't an array of strings
	    if (orelse
	           { typeof(stacklist[key]) != Array_Type }
		   { _typeof(stacklist[key]) != String_Type }
	       ) {
		usage(usage_msg);
	    }
	}
    }

    % Initialize count of history lines
    variable histnum = old_histnum;

    % Open parameter file for reading; return on failure

    variable parfile;		% Parameter file object

    parfile = paramopen(progname,"r");
    if (parfile == NULL)  return (NULL,NULL);


    %
    % Write tool name and date/time history lines
    %

    variable histstr;		% Content of current history line
    variable tparts;		% Time/date data from "gmtime"

    tparts = gmtime(_time());

    % Create history string in format
    %   "toolname   YYYY-MM-DDTHH:MM:SS"

    histstr = sprintf("%s   %04d-%02d-%02dT%02d:%02d:%02d",progname,
	tparts.tm_year+1900,tparts.tm_mon+1,tparts.tm_mday,tparts.tm_hour,
	tparts.tm_min,tparts.tm_sec);

    % Create history lines and push them onto the stack
    histnum = create_history_lines("TOOL",histstr,histnum);


    %
    % Write parameter history lines
    %

    variable name;		% Name of current parameter
    variable value;		% Value of current parameter

    % Process each parameter listed in parameter file

    foreach(plist_names(parfile)) {
	name = ();

	% If "paramlist" was provided and contains a value for this
	% parameter, store that value; otherwise, get value from
	% parameter file

	if (andelse {paramlist != NULL} {assoc_key_exists(paramlist,name)})
	    value = paramlist[name];
	else
	    value = pget(parfile,name);

	% Create history string
	histstr = sprintf("%s=%S",name,value);

	% Create history lines and push them onto stack
        histnum = create_history_lines("PARM",histstr,histnum);

	% If "stacklist" was provided and contains an entry for this
	% parameter, create history lines for each stack element

	if (andelse {stacklist != NULL} {assoc_key_exists(stacklist,name)}) {
	    foreach (stacklist[name]) {
		% Get history string
		histstr = ();

		% Create history lines and push them onto stack
                histnum = create_history_lines("STCK",histstr,histnum);
	    }
	}
    }

    % Close parameter file
    paramclose(parfile);


    %
    % Create and return history array
    %

    variable history;		% Array of history lines
    variable histsize;		% Size of history array
    variable i;			% Loop counter

    % Set size to accommodate all new history lines
    histsize = histnum - old_histnum;

    % Create history array
    history = String_Type[histsize];

    % Add each item on the stack to the history array (starting
    % at the end and moving backwards to get the order right)

    _for(histsize-1, 0, -1) {
	i = ();
	history[i] = ();
    }

    % Return the new HISTNUM and history array
    return (history,histnum);
}


public define cxchistory_write_history()
%
% Given file, histarray, and histnum (see descriptions below), writes history
% information to header of specified FITS file.  Returns zero if successful
% and a FITS error code on failure.
%
{
    %
    % Store and validate arguments
    %

    variable file;		% File name or ciao_Fits_File_Type object
    variable history;		% Array of strings containing history lines
    variable histnum = NULL;	% Value of HISTNUM keyword

    % Usage message
    variable usage_msg = "\n  status = " + _function_name() +
        "(file,histarray[,histnum])\n\nParameters:\n" +
	"  file      : name of file to which to write history or\n" +
	"              ciao_Fits_File_Type object open for writing\n" +
	"  histarray : array of strings containing history lines\n" +
	"  histnum   : value of HISTNUM keyword\n";

    % Get arguments

    switch (_NARGS)
	{ case 2 : (file,history) = (); }
	{ case 3 : (file,history,histnum) = (); }
	{ usage(usage_msg); }

    % Validate arguments

    if (orelse
	   % "file" must be a string or ciao_Fits_File_Type object
	   { andelse
	       { typeof(file) != String_Type }
	       { typeof(file) != ciao_Fits_File_Type }
	   }

	   % "history" must be an array of strings
	   { typeof(history) != Array_Type }
	   { _typeof(history) != String_Type }

	   % "histnum" must be NULL or an integer
	   { andelse
	       { histnum != NULL }
	       { typeof(histnum) != Integer_Type }
	   }
       ) {
	% Die if any argument was of incorrect type
        usage(usage_msg);
    }


    %
    % Write history to specified file
    %

    variable fp;	% ciao_Fits_File_Type pointer
    variable status;	% Status value from _fits* commands

    % If necessary, open file for writing

    if (typeof(file) == ciao_Fits_File_Type) {
	fp = file;
    } else {
        status = _fits_open_file(&fp,file,"w");
        if (status)  return status;
    }

    % If "histnum" was provided, update HISTNUM keyword

    if (histnum != NULL) {
        status = _fits_update_key(fp,"HISTNUM",histnum,NULL);
        if (status)  return status;
    }

    % Write history strings to file

    variable histstr;	% Current history string

    foreach(history) {
	histstr = ();
	status = _fits_write_history(fp,histstr);
        if (status)  return status;
    }

    % If we opened "fp", close it

    if (typeof(file) != ciao_Fits_File_Type) {
	% The version of _fits_close_file() in the cfitsio
	% module returns a value; the version in VARMM (and
	% older versions of ISIS) does not
#ifexists _cfitsio_module_version
	status = _fits_close_file(fp);
        if (status)  return status;
#else
        _fits_close_file(fp);
#endif
    }

    % Indicate success
    return 0;
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% If provide() is available, register the package
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Currently, this causes an error message when run after importing
%% the sherpa module

%#ifexists provide
%provide("cxchistory");
%#endif


