	  %
	  % check_cols( a, b, colname, scale [, tol ] )
	  %
	  % check that two columns are the same, apart from a scaling factor,
	  % to within a given tolerance (defaults to 0)
	  %
	  % ie fail if any element of 
	  %    abs( a.<colname> * scale - b.<colname> )
	  % is greater than tol
	  %
	  % a and b can be either a structure, containing the given column
	  % as a field (ie the return value of a varmm readfile() command) or
	  % the name of a file which contains this column
	  %
	  define check_cols () {
	
	    variable usage_str =
	      "check_cols(file1|struct1,file2|struct2,colname,scale[,tol])";
	
	    % see if correct number of arguments were supplied 
	    variable in1, in2, colname, scale, tol;
	    if ( _NARGS == 5 ) {
	      ( in1, in2, colname, scale, tol ) = ();
	    } else if ( _NARGS == 4 ) {
	      ( in1, in2, colname, scale ) = ();
	      tol = 0;
	    } else {
	      % clear the stack of the arguments sent to this function
	      % so other commands are not affected
	      if ( _NARGS != 0 ) _pop_n( _NARGS ); 
	      usage( usage_str );
	    }
	
	    % The first two parameters can be either a filename (String_Type)
	    % or a structure (Struct_Type). If the former then we need
	    % to read in the data from the file
	    %
	    % if we have a structure, then do nothing, otherwise read in
	    % the column from the specified file, using the data model
	    % to make sure we only get the required column
	    %
	    variable cols1, cols2;
	
	    % first file/structure
	    if ( typeof(in1) == Struct_Type ) {
	      cols1 = in1;
	    } else if ( typeof(in1) == String_Type ) {
	      cols1 = readfile( in1 + "[cols " + colname + "]" );
	      if ( cols1 == NULL )
	        error( "Unable to read the " + colname + " column from " 
	          + in1 );
	    } else usage( usage_str );
	
	    % second file/structure
	    if ( typeof(in2) == Struct_Type ) {
	      cols2 = in2;
	    } else if ( typeof(in2) == String_Type ) {
	      cols2 = readfile( in2 + "[cols " + colname + "]" );
	      if ( cols2 == NULL )
	        error( "Unable to read the " + colname + " column from " 
	          + in2 );
	    } else usage( usage_str );
	
	    % get references to the columns (assume fields exist)
	    variable c1 = get_struct_field( cols1, colname );
	    variable c2 = get_struct_field( cols2, colname );
	
	    % is the absolute difference > the tolerance anywhere?
	    variable absdiff = abs( scale * c1 - c2 );
	    variable index = where( absdiff > tol );
	
	    % if there are any elements in index, then the 2 columns do 
	    % not match to within the tolerance
	    if ( length(index) != 0 ) {
	      % find out the max value
	      variable maxval = max( absdiff[index] );
	
	      error( "2 columns do not agree within a tolerance of " + 
	        string(tol) + "\n  max diff = " + string(maxval) );
	    }
	  
	  } % check_cols()
		      
