%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% sherpa_paramset.sl
%
% Version 0.1 (2004/01/28)
%
% Author:  Christopher J. Stawarz (cstawarz@cfa.harvard.edu)
%
% Purpose:
%   Provides high-level functions for setting Sherpa model parameters (based on
%   interactive or command-line input from the user) from within a S-Lang
%   script.  Also provides convenience wrapper functions for 'ERASE ALL' and
%   'ERASE <modelname>'.
%
% Public Functions:
%   prompt_for_model_param()
%   process_model_param_expr()
%   erase_all()
%   erase_model()
%
% Message and Error Functions:
%   By default, the package will use message() for debugging/informational
%   messages (verbosity level is controlled by sherpa_paramset->verbose;
%   value of 0 means no messages, 5 means maximum messages) and verror()
%   for error messages.
%
%   However, it is possible to customize message/error handling by assigning
%   function references to the variables sherpa_paramset->message_func and
%   sherpa_paramset->error_func, respectively.  If these variables are set,
%   then the arguments to the private functions do_message() and do_error()
%   will be passed to the specified handlers, and the default functions will
%   not be called.
%
%   If used, the function associated with sherpa_paramset->message_func must
%   accept one or two arguments (a message string, or an integer verbosity
%   value and message string).  The function associated with
%   sherpa_paramset->error_func must accept one argument (an error message
%   string).
%
% History:
%   2004/01/28 - initial version
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


implements("sherpa_paramset");


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Message and error functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Message verbosity level
static variable verbose = 0;

% Package name to report in error messages
private variable package_name = current_namespace();

% References to external message and error functions
static variable message_func = NULL;
static variable error_func = NULL;


private define do_message()
%
% Display message string "msg".  If optional integer "vlevel" is provided,
% display message only if verbose >= vlevel.
%
% Alternatively, if "message_func" is not NULL, pass all arguments to
% the function it refers to and return.  (That is, use a user-specified
% external message handler.)
%
{
    if (message_func != NULL) {
	variable args = __pop_args(_NARGS);
	@message_func(__push_args(args));
	return;
    }

    variable vlevel = 0;
    variable msg;

    switch (_NARGS)
      { case 1 : msg = (); }
      { case 2 : (vlevel,msg) = (); }

    if (verbose >= vlevel)  message(msg);
}


private define do_error()
%
% Issue error message "msg" in standard format and throw error. 
%
% Alternatively, if "error_func" is not NULL, pass all arguments to
% the function it refers to and return.  (That is, use a user-specified
% external error handler.)
%
{
    if (error_func != NULL) {
	variable args = __pop_args(_NARGS);
	@error_func(__push_args(args));
	return;
    }

    variable msg = ();
    verror("# %s: ERROR: %s",package_name,msg);
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Private utility functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


private define bad_parspec_msg(parname,msg)
%
% Display formatted parameter-specification error message.  Used by
% parse_model_param_string().
%
{
    do_message(sprintf("%s : %s",parname,msg));
}


private define parse_model_param_string(parname,parstr)
%
% Given "parname", a model parameter name, get the settings for the
% parameter by processing string "parstr", which is an expression of the
% form
%
%   [<value>][:[<min>][:[<max>][:[<delta>]]]][,+/-1]
%
% where value, min, max, and delta are the parameter value, minimum value,
% maximum value, and step size, respectively, and the +/-1 indicates that
% the parameter should be thawed/frozen.  Note that this is basically the
% syntax described in 'ahelp create', except for the addition of +1 to
% indicate thawing.
%
% Once the string has been processed, the function returns a parameter
% structure with the fields set appropriately, suitable for immediate
% passing to set_par().
%
{
    % Get parameter structure
    variable par = get_par(parname);

    % Make sure "parname" matches one and only one parameter

    if (par == NULL) {
	bad_parspec_msg(parname,"unknown model parameter");
	return NULL;
    } else if (typeof(par) == Array_Type) {
	bad_parspec_msg(parname,"multiple matches");
	return NULL;
    }

    % Remove leading/trailing spaces
    parstr = strtrim(parstr);

    % If parstr is wrapped in parentheses (as it may be if it was given
    % as a stack element in a parameter file), remove them

    variable len = strlen(parstr);
    if ((parstr[0] == '(') and (parstr[len-1] == ')')) {
	parstr = parstr[[1:len-2]];
    }

    % If value is a string (e.g. a filename for an instrument
    % model), set value to "parstr" and return

    if (typeof(par.value) == String_Type) {
	par.value = parstr;
	return par;
    }


    %
    % Parse the expression
    %

    variable fields = strchop(parstr,',',0);
    len = length(fields);

    if (len > 2) {
	bad_parspec_msg(parname,"too many commas in specification");
	return NULL;
    } else if (len == 2) {
	variable thawflag = strtrim(fields[1]);

	if (thawflag != "") {
	    if (_slang_guess_type(thawflag) == String_Type) {
		bad_parspec_msg(parname,"freeze/thaw flag must be numeric");
		return NULL;
	    }

	    % If thawflag is negative, freeze the parameter
	    par.frozen = int(atof(thawflag) < 0);
	}
    }

    fields = strchop(fields[0],':',0);
    len = length(fields);

    if (len > 4) {
	bad_parspec_msg(parname,"too many fields in specification");
	return NULL;
    }

    variable value;
    variable i;

    _for (0, len-1, 1) {
	i = ();

	fields[i] = strtrim(fields[i]);

	% If field is empty, use the current value
	if (fields[i] == "")  continue;

	if (_slang_guess_type(fields[i]) == String_Type) {
	    bad_parspec_msg(parname,"parameter fields must be numeric");
	    return NULL;
	}

	value = atof(fields[i]);

	switch (i)
	  { case 0 : par.value = value; }
	  { case 1 : par.min = value; }
	  { case 2 : par.max = value; }
	  { case 3 : par.delta = value; }
    }

    % Check that value is within parameter limits

    if (par.value < par.min) {
	bad_parspec_msg(parname,"value below minimum");
	return NULL;
    }

    if (par.value > par.max) {
	bad_parspec_msg(parname,"value above maximum");
	return NULL;
    }

    % Return the parameter structure
    return par;
}


private define print_param_settings(verbval,par)
%
% If verbose > verbval, print settings in parameter structure "par".
%
{
    % For string parameters, just print value
    if (typeof(par.value) == String_Type) {
        do_message(verbval,sprintf("value  = %s",par.value));
	return;
    }

    do_message(verbval,sprintf("value  = %g",par.value));
    do_message(verbval,sprintf("min    = %g",par.min));
    do_message(verbval,sprintf("max    = %g",par.max));
    do_message(verbval,sprintf("delta  = %g",par.delta));
    do_message(verbval,sprintf("frozen = %d",par.frozen));
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Primary public functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


public define prompt_for_model_param()
%
% Given "parname", the name of a model parameter, prompt user for the
% parameter's settings and then set them.  Parameter specification is as
% described for parse_model_param_string().  Note that the function will not
% exit until the user provides valid input.
%
{
    variable parname;

    switch (_NARGS)
      { case 1 : parname = (); }
      { usage(sprintf("%s(paramname)",_function_name())); }

    parname = string(parname);

    % Die if we're not reading from a terminal
    !if (isatty(stdin))
	do_error("standard input is not a terminal");

    % Make sure parname matches one and only one parameter

    variable par = get_par(parname);

    if (par == NULL)
	do_error("unknown model parameter: " + parname);

    if (typeof(par) == Array_Type)
	do_error("multiple matches for parameter " + parname);

    % Print current settings

    do_message(2,sprintf("\nCurrent settings for parameter %s:",
      par.name));
    print_param_settings(2,par);
    do_message(2,"");


    %
    % Get user input
    %

    variable input = NULL;

    while (input == NULL) {
	if (typeof(par.value) == String_Type)
            () = printf("%s parameter value [%s] ",par.name,par.value);
	else
            () = printf("%s parameter value [%g] ",par.name,par.value);

        if (-1 == fgets(&input,stdin)) {
	    % Check whether failure occurred because stdin was closed, as
	    % we don't want to throw an error if the user typed Ctrl-D
	    !if (feof(stdin))
	        do_error("unable to read from standard input");

	    % If the user typed Ctrl-D and nothing else, input will be
	    % NULL
	    if (input == NULL)  input = "";
        }

	% Parse the input string.  If the parse fails, input will be
	% NULL, and the loop will repeat.
	input = parse_model_param_string(parname,input);
    }

    % Copy parameter struct in input to par
    par = input;

    % Fail if we can't set the parameter
    !if (set_par(par))
        do_error("unable to set model parameter " + parname);

    % Print new settings
    do_message(4,sprintf("\nUpdated settings for parameter %s:",par.name));
    print_param_settings(4,par);
}


public define process_model_param_expr()
%
% Parse expression of the form
%
%   <parname>=[(][<value>][:[<min>][:[<max>][:[<delta>]]]][,+/-1][)]
%
% and set parameter "parname" appropriately.  The function will throw an
% error if anything goes wrong.
%
{
    variable paramexpr;

    switch (_NARGS)
      { case 1 : paramexpr = (); }
      { usage(sprintf("%s(paramexpr)",_function_name())); }

    paramexpr = string(paramexpr);


    %
    % Parse paramexpr
    % 

    variable parts;
    variable name;
    variable value;
    variable par;

    parts = strchop(paramexpr,'=',0);

    % Make sure there's exactly one '='
    if (length(parts) != 2)
        do_error("bad model parameter expression: " + paramexpr);
	
    name = strtrim(parts[0]);
    value = parts[1];

    % Die if no parameter name was given
    if (name == "")
        do_error("missing model parameter name: " + paramexpr);

    % Make sure "name" matches one and only one parameter

    par = get_par(name);

    if (par == NULL)
	do_error("unknown model parameter: " + name);

    if (typeof(par) == Array_Type)
	do_error("multiple matches for parameter " + name);

    % Parse "value"; die if there's an error
    par = parse_model_param_string(name,value);
    if (par == NULL)
        do_error("unable to process model parameter expression");

    % Set the parameter
    !if (set_par(par))
        do_error("unable to set model parameter " + name);
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Public convenience functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


public define erase_all()
%
% Wrapper for "ERASE ALL"
%
{
    () = sherpa_eval("ERASE ALL");
}


public define erase_model()
%
% Wrapper for "ERASE <modelname>"
%
{
    variable model;

    switch (_NARGS)
      { case 1 : model = (); }
      { usage(sprintf("status = %s(modelname)",_function_name())); }

    model = string(model);

    if (length(strtok(model)) != 1)  return 0;

    % Make sure user isn't trying to ERASE ALL, DATA, or BACK
    variable word = strup(strtrim(model));
    if ((word == "ALL") or (word == "DATA") or (word == "BACK"))
	return 0;

    % Try to erase the model
    if (-1 == sherpa_eval(sprintf("ERASE %s",model)))  return 0;

    return 1;
}


