%
% $Id: sherpa_utils.sl,v 1.26 2004/11/02 15:53:37 dburke Exp $
%
% Aim:
%  Provide a number of S-Lang functions useful for
%  using/interacting with Sherpa.
%
% Requires:
%   CIAO 3.2
%
% Threads:
%  http://cxc.harvard.edu/ciao/threads/calc_kcorr/
%  http://chandra.ledas.ac.uk/ciao/threads/calc_kcorr/
%
% Usage:
%  See http://cxc.harvard.edu/ciao/download/scripts/ or
%  http://chandra.ledas.ac.uk/ciao/download/scripts/
%  for instructions on how to install S-Lang scripts.
%
%  After installation, the script can be loaded using:
%
%  - from the Sherpa shell
%
%      sherpa> require( "sherpa_utils" )
%
%  - in a S-Lang script
%
%      require( "sherpa_utils" );
%
% Documentation:
%  
%  - ahelp sherpa_utils
%  - ahelp <function name> (e.g. 'ahelp groupByCounts')
%  - http://cxc.harvard.edu/ciao/ahelp/sherpa_utils.html
%  - http://chandra.ledas.ac.uk/ciao/ahelp/sherpa_utils.html
%
% Version 1.26 (CIAO 3.2) changes:
%   - the script can now be loaded via 'require("sherpa_utils");'
%   - uses require() to load in the Sherpa and group modules
%
% Version 1.24 (CIAO 3.1) changes:
%   - added the groupByXXX() routines.
%
% CIAO 3.0.2 changes (visible to the user):
%   - added get_source_components()
%
%   - updated set_dataspace() to use set_axes() call since
%     bugs have been fixed
%
% CIAO 3.0 changes (visible to the user):
%   - Removed the following functions:
%       is_stuct_field()
%       get_modelname()
%       get_modelinfo()
%       filter_model()
%       convert2model()
%       read_model()
%       sumup_model()
%       calc_flux()    MAY BE RESTORED?
%       print_cflux()  ditto
%       *_help() routines
%
%   - since read_model() is no longer available, the first parameter
%     of calc_kcorr() must now be the dataset number
%
%   - added an ahelp file for the package (takes the place of the
%     *_help() routines)
%
% CIAO 3.0 changes (internal):
%   - We no longer uses the sherpa_utils namespace
%   - Removed all the SpectralModel* structure definitions since there
%     are now much easier ways to get the information.
%   - Removed the list of additive vs multiplicative XSPEC models (the
%     _xspec_models associative array)
%
% Variables:
%   This package defines 2 variables that are 'exported' into the global
%   namespace:
%     _sherpa_utils_version_string
%     _sherpa_utils_version
%   they define the version of the file, and are analogous to the
%   _slang_version and _slang_version_string variables, except
%   that they are not read only, and the numbering scheme is
%   a.b not a.b.c
%
%   A number of other variables are available within the "sherpa_utils"
%   namespace, but please do not mess with them unless you understand this
%   code.
%
% Functions:
%   More details on these functions are available in the
%   "User-callable code" section below. The routines will print out a
%   usage message if called incorrectly, so calling the function with no
%   parameters should provide helpful information.
%
%     sherpa> set_dataspace
%     Usage:
%      set_dataspace( dnum, elo, ehi, estep )
%
%     set_dataspace;
%
% Quick listing:
%
% set_dataspace( dnum, elo, ehi, estep )
%
%   sets the dataspace for the given slot/# (number)
%   to be elo:ehi:estep - in HISTOGRAM mode
%
% cpts = get_source_components();
% cpts = get_source_components( dnum );
%
%   returns an array containing the names of the source
%   components used for dataset number dnum (defaults to 1).
%
% kcorr = calc_kcorr( dnum, z, elo, ehi )
% kcorr = calc_kcorr( dnum, z, obslo, obshi, restlo, resthi )
%
%   Calculates the k correction for the model spectrum
%   when at redshift z for the energy range elo to ehi,
%   or from the observed energy range obslo to obshi to the
%   rest frame energy range restlo to resthi
%
% Grouping routines:
%   simple routines to group your data from within Sherpa
%
% groupByCounts( dset, numCounts )
%
%   groups dataset # dset so that there are
%   at least numCounts counts per bin
%
% groupBySNR( dset, minSNR );
%
%   groups dataset # dset so that the minimum signal-to-noise
%   ratio is at least minSNR
%
% groupAdaptively( dset, minCounts );
%
%   adaptive grouping scheme to achieve at least minCounts per
%   bin. It is slightly different to groupByCounts() since it
%   does not work "left to right" - i.e. march through the
%   array and group bins until the counts criterion is exceeded -
%   instead it works by numbed of counts. See the
%   description of grpAdaptive() for more details.
%
% Note:
%   This algorithm is designed to enhance the significance of
%   emission rather than absorption features. 
%
% groupAdaptiveSNR( dset, minSNR );
%
%   adaptive grouping scheme to achieve a signal-to-noise
%   of at least minSNR per bin.
%   It is slightly different to groupBySNR() since it
%   does not work "left to right" - i.e. march through the
%   array and group bins until the counts criterion is exceeded -
%   instead it works by numbed of counts. See the
%   description of grpAdaptive() for more details.
%
% Note:
%   This algorithm is designed to enhance the significance of
%   emission rather than absorption features. 
%
% Notes on the grouping routines:
%  - calculations are made on the data without subtracting
%    the background.
%  - want to be able to add extra constraints such as:
%    - when grouping by SNR use these values as the errors,
%      not sqrt(ncounts)
%    - max number of bins to group
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Internal code
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

require("sherpa");
require("group");

% Check the version
% - we ignore the pe-CIAO 3.0 case here since the require() line
%   above will only work for CIAO 3.2 and later
%
if ( 30200 > sherpa_version )
  error("\n*** ERROR: sherpa_utils.sl requires Sherpa >= 3.2.\n" );

% Usage:
%  ( strval, intval ) = _get_version_info( rcs_version_string )
%
% Aim:
%   Parse the RCS version string into string and integer values
%
%   rcs_version_string should be "@Revision@" [where @ is replaced
%   by $ and the whole string has been expanded by RCS].
%
private define _get_version_info() {
  if ( 1 != _NARGS ) {
    variable cleanup = __pop_args( _NARGS );
    usage( "(strval,intval) = _get_version_info( rcs_version_string )\n" +
      " where the input string looks like '$Revision: 1.26 $'.\n" );
    return ( NULL, NULL );
  }

  variable rcs = ();

  % strip out the excess text
  variable version_string = substr( rcs, 12, strlen(rcs)-13 );

  % calculate a numeric value
  variable version_int = 0;
  variable i = 0;
  foreach ( strchopr( version_string, '.', 0 ) ) {
    variable num = ();
    version_int += ( integer(num) * 10^i );
    i+=2;
  }

  % return
  return ( version_string, int(version_int) );

} % _get_version_info()

% set up the version variables from the RCS Revision id
% - unlike the slang version variables (_slang_version and _slang_version_string)
%   these variables are writeable
%
public variable _sherpa_utils_version_string, _sherpa_utils_version;
( _sherpa_utils_version_string, _sherpa_utils_version ) =
  _get_version_info( "$Revision: 1.26 $" );

%% local routines and variables
%

#if false

% The _sherpa_utils_verbose flag determinse whether the routines
% print extra information to stdout during processing. If
% set to 0 then this does not happen, if set to 1 it does.
% Behaviour with other values is currently un-defined.
%
% Note - we only set the variable to 0 the first time the file
% is loaded. This way a user does not have to reset the variable
% if they re-load the file
%
public variable _sherpa_utils_verbose;
!if ( __is_initialized(&_sherpa_utils_verbose) )
  _sherpa_utils_verbose = 0;

% prints messages to stdout
%  - msg can be an array
%  - each line of text is preceeded by
%      "sherpa_utils.sl v<version>: "
%    and a new-line is appended to it
%
private define dbg(msglist) {
  if ( 0 != _sherpa_utils_verbose ) {
    if ( Array_Type != typeof(msglist) )
      msglist = [ msglist ];
    foreach ( msglist ) {
      variable msg = ();
      () = fprintf( stdout, "sherpa_utils.sl v%s: %s\n",
		    _sherpa_utils_version_string, msg );
    } % foreach: msglist
  } % if 0 != verbose
} % dbg()

#else

% we are not debugging so do not do anything
%
private define dbg(msglist) {}

#endif

%
private define myerr(errstr) {
  verror("\n***** error from sherpa_utils.sl version %s *****\n%s\n*****\n",
	 _sherpa_utils_version_string, errstr );
}

% execute a sherpa command, or list of commands, exiting on error
%
private define seval( commands ) {
  if ( Array_Type != typeof( commands ) ) commands = [ commands ];
  foreach ( commands ) {
    variable comm = ();
    if ( 0 != sherpa_eval( comm ) )
      myerr( "Unable to execute the following sherpa command\n" +
	     comm );
  }
} % seval()

% too lazy to type '() = printf(...);'!
%
private define mypf () {
  variable args = __pop_args (_NARGS);
  () = printf( __push_args (args) );
} % mypf()

%
% When a dataset is grouped we want to know what filters
% were specified, so that we can re-apply them after
% grouping (the grouping happens using the *un-filtered* data)
% However, those filters that rely on bin/channel number
% can *NOT* be re-applied (the mapping from bin/channel number
% is unlikely to be preserved by the new grouping). We list here,
% in an associative array, those filters we ignore when
% re-applying the filters
%
% We use an associative array to make checking fot it easier later
% on (ie 'if ( assoc_key_exists(forbidden_filters,filter) ) ...')
% - the value stored in the array is not actually used
%
private variable forbidden_filters = Assoc_Type[];
forbidden_filters[ "filter"   ] = 1;
forbidden_filters[ "bins"     ] = 1;
forbidden_filters[ "channels" ] = 1;

%
% Usage:
%   doGrouping( dset, funcref, funcval );
%
% Aim:
%   applies the requested grouping scheme (the
%   function reference funcref whose parameter value is funcval)
%   to the given dataset.
%
% Method:
%   - check for background status (ie background subtracted or not)
%     and then unsubtact the background if necessary
%   - find all the filters applied to the dataset
%     (at least those set by sherpa commands rather than the set_filter()
%      routine)
%   - remove any filters
%   - ungroup the dataset (if grouped)
%   - get the data
%   - group the data using the supplied function and parameter(s)
%   - apply the grouping (issues with setting quality at the moment)
%   - re-apply the filters if there are any valid ones
%
% Notes:
%  - may want to allow multiple arguments to the function
%    e.g. to allow an error column for grpSnr()
%  - this is a private utility routine, users are expected
%    to use the public routines described below
%
private define doGrouping( dset, funcref, funcval ) {

  dbg( "Grouping dataset " + string(dset) +
       " - grouping argument = " + string(funcval) );

  % We use get_data() as a way of determining whether dataset number
  % 'dset' is valid (ie contains data). We do not actually use the
  % values returned by this call since we need to call ungroup first
  %
  if ( NULL == get_data(dset) )
    verror( "Unable to find any data for dataset %d\n", dset );

  % ignore error messages for subraction/grouping here
  % - we clear all the filters to try and avoid loads of warning messages
  %   about how filters will be ignored when we do things like ungroup
  %
  variable bgnd_flag = is_subtracted(dset);
  if ( bgnd_flag ) {
    () = set_unsubtract(dset);
    dbg( "dataset is background subtracted" );
  } else {
    dbg( "dataset is not background subtracted" );
  }

  variable filters = get_filter_expr(dset);
  if ( 1 != set_notice_all(dset) )
    verror( "Unable to clear the filters for source %s\n", dset );

  % We only call 'ungroup' if the data was grouped so as to avoid
  % the "Error: data cannot be ungrouped." message which might
  % confuse users. We assume that if all the group elements == 1
  % then the file is not grouped.
  %
  variable groups = get_groups(dset);
  if ( NULL == groups )
    verror( "Unable to access group column from dataset %d.\n", dset );
  if ( sum(groups) != length(groups) )
    () = sherpa_eval( "ungroup " + string(dset) );

  variable data = get_data(dset);
  if ( NULL == data )
    verror( "Unable to find any data for dataset %d\n", dset );
  dbg( "ungrouped dataset contains " +
       string(length(data)) + " bins" );

  % do the actual grouping
  %
  variable grp, qual;
  ( grp, qual ) = @funcref( data, funcval );
  dbg( [
	"after grouping -> " +
	string(sum(typecast(grp>0,Int_Type))) + " bins",
	"and " +
	string(sum(typecast(qual>0,Int_Type))) +
	" bins have a non-zero quality value"
	] );

  % we try to be clever and only call set_quality() if the quality
  % array contains any non-zero elements (we rely on the fact the
  % elements are >=0 for this test)
  %
  !if ( set_groups(dset,grp) )
    verror( "Unable to set the grouping for dataset %d (grouping parameter = %g)\n",
            dset, funcval );
  if ( andelse
    { sum(qual) > 0 }
    { 1 != set_quality(dset,qual) } )
       verror("Unable to set the quality flags for dataset %d\n", dset );

  % re-apply the filters - EXCEPT for those defined in bins/channels
  %
  foreach ( filters ) {
    variable str = ();
    variable words = strtok( str );
    variable match = strlow( words[3] );
    if ( assoc_key_exists( forbidden_filters, strlow(words[3]) ) ) {
      print( "Note: ignoring filter expression '" + str + "'" );
      continue;
    }
    
    % apply the filter
    dbg( "applying filter: '" + str + "'" );
    if ( 0 != sherpa_eval( str ) )
      verror( "Unable to re-filter spectrum using:\n  '%s'\n", str );

  } % foreach: filters

  % sort out the state of the background
  %
  if ( bgnd_flag ) {
    dbg( "subtracting the bacgkround from the data" );
    () = set_subtract(dset);
  }

  return;

} % doGrouping()


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% User-callable code
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% set_dataspace()

public define set_dataspace() {

  variable usage_str =
    "\n  " + _function_name + "( dnum, elo, ehi, estep )\n";
  if ( 4 != _NARGS ) {
    () = __pop_args( _NARGS );
    usage( usage_str );
  }
  variable number, elo, ehi, estep;
  ( number, elo, ehi, estep ) = ();

  variable s = struct { lo, hi };
  variable axis = [elo:ehi+estep:estep];
  s.lo = axis[[0:-2]];
  s.hi = axis[[1:-1]];
  !if ( set_axes( number, s ) )
    mypf( "Error - set_dataspace(): Unable to set axis for dataset %d to %g:%g:%g\n",number,elo,ehi,estep );

} % set_dataspace()

%
% Usage:
%   models = get_source_components( [ dnum ] );
%
% Aim:
%   Returns a string array which contains the names of the
%   model components used in the source expression for
%   dataset number dnum (defaults to 1 if not given).
%   So, if the source expression was set to
%     xsmekal[clus] * xsphabs[gal]
%   then get_source_expr() would return "(clus * gal)"
%   and this routine would return a 2-element array
%   with values "clus" and "gal".
%
% Notes:
%   Returns NULL if a source expression is not defined
%   for dataset dnum.
%
public define get_source_components() {

  variable dnum;
  switch ( _NARGS )
    { case 1: dnum = (); }
    { case 0: dnum = 1; }
    { usage( _function_name + "( [dnum] );" ); }
  
  % find the source expression
  % (also checks that dnum is valid)
  %
  variable source_expr = get_source_expr(dnum);
  if ( orelse { NULL == source_expr } { "" == source_expr } )
    return NULL;

  variable err_str = sprintf( "Internal error (%s): unable to parse 'source %d = %s'\n",
			      _function_name, dnum, source_expr );

  % parse the source expression to get the names of the
  % model components
  %
  variable models = strtok( source_expr, " ()+-*/" );
  if ( orelse { NULL == models } { 0 == length(models) } )
    verror( err_str );

  variable indx = where( models != "+" and models != "-" and models != "*" and models != "/" );
  if ( 0 == length(indx) )
    verror( err_str );
  
  return models[indx];

} % get_source_components()

%% calc_kcorr()

public define calc_kcorr () {
  variable usage_str = 
    "\n  kcorr = " + _function_name +
    "( dnum, z, obslo, obshi [,restlo,resthi] )\n\n" +
    "where dnum is the number of the source model to use (an integer >=1)\n" +
    "z is a scalar or array (with values >=0), and the energies are scalars.\n";

  % sort out what arguments the user has supplied
  variable src, z, olo, ohi, rlo, rhi;
  switch ( _NARGS )
  {
    case 6:
      ( src, z, olo, ohi, rlo, rhi ) = ();
  }
  {
    case 4:
      ( src, z, olo, ohi ) = ();
      rlo = olo;
      rhi = ohi;
  }
  {
    () = __pop_args( _NARGS );
    usage( usage_str );
  }

  % we convert a scalar z into an array, to make the following code
  % easier, but retain a flag so that we can return a scalar value
  % at the end
  %
  variable zflag = 0;
  if ( typeof(z) != Array_Type ) {
    z = [ z ];
    zflag = 1;
  }

  % safety checks
  %
  % - is there a model?
  % - are the redshifts all sensible?
  % - are all the energies sensible (lo < hi and the energy grid of the model) 
  %
  if ( NULL == get_source_expr(src) ) {
    mypf( "Error - calc_kcorr():\n could not find a source model for dataset number %d\n", src );
    return NULL;
  }
  if ( 0 != sum(z < 0) ) {
    mypf( "Error - calc_kcorr():\n At least one redshift was <= 0.\n" );
    return NULL;
  }
  if ( olo <= 0 or rlo <= 0 or ohi <= olo or rhi <= rlo ) {
    mypf( "Error - calc_kcorr():\n The energy range(s) are not 0<=elo<ehi.\n" );
    return NULL;
  }
  variable paxes = get_photon_axes(src);
  if ( NULL == paxes ) {
    mypf( "Error - calc_kcorr():\n could not find axis information for dataset number %d\n", src );
    return NULL;
  }
  if ( NULL != paxes.mid ) {
    mypf( "Error - calc_kcorr():\n it appears that dataset number %d is not binned\n", src );
    return NULL;
  }
  variable emin = paxes.lo[0];
  variable emax = paxes.hi[-1];
  if ( rlo < emin or rhi > emax ) {
    mypf( 
      "Error - calc_kcorr():\n The model is defined over the energy range " +
      "%f to %f keV\n which does not fully overlap the rest-frame energy band of\n" +
      " %f to %f keV\n", emin, emax, rlo, rhi );
    return NULL;
  }
  variable zmin = min(z);
  if ( olo*(1.0+zmin) < emin ) {
    mypf( 
      "Error - calc_kcorr():\n The model is defined over the energy range " +
      "%f to %f keV\n which does not fully overlap the observed-frame energy band of\n" +
      " %f to %f keV at a redshift of %f.\n", rlo, rhi, emin, emax, zmin );
    return NULL;
  }
  variable zmax = max(z);
  if ( ohi*(1.0+zmax) > emax ) {
    mypf( 
      "Error - calc_kcorr():\n The model is defined over the energy range " +
      "%f to %f keV\n which does not fully overlap the observed-frame energy band of\n" +
      " %f to %f keV at a redshift of %f.\n", rlo, rhi, emin, emax, zmax );
    return NULL;
  }

  variable nz = length(z);
  variable zplus1 = 1.0 + z;

  % calculate the rest frame "flux"
  %
  variable rest = get_eflux( src, [rlo, rhi] );
  if ( NULL == rest )
    mypf( "Error - calc_kcorr():\n unable to calculate rest-frame flux for dataset %d\n", src );
  variable flux_rest = rest.value;

  % loop through each redshift repeating the calculation for the observed frame
  %
  variable kcorr = z * 0.0;
  variable l;
  _for ( 0, nz-1, 1 ) {
    l = ();
    variable obs = get_eflux( src, [olo, ohi]*zplus1[l] );
    if ( NULL == obs )
      mypf( "Error - calc_kcorr():\n unable to calculate redshifted flux for dataset %d at z=%f\n", src, zplus1[l]-1 );
    variable flux_obs = obs.value;

    kcorr[l] = flux_rest / flux_obs;
  }

  % return the k correction
  if ( zflag ) return kcorr[0];
  else         return kcorr;

} % calc_kcorr()

%%%
%%% the grouping routines
%%%

% used to create the 'usage' message for each function
%
private variable usage_filter_str =
    "\nAny filters given in energy/wavelength units will be restored after the\n" +
    "data has been grouped, but those filters defined using any of:\n  " +
    strjoin( assoc_get_keys(forbidden_filters), " " ) + "\n" +
    "will be lost.\n";

%
% Usage:
%   groupByCounts( dset, numCounts );
%
% Aim:
%   groups dataset # dset so that there are
%   at least numCounts counts per bin
%
public define groupByCounts() {
  variable usage_str =
    _function_name + "( [dset], numCounts );\n\n" +
    "Groups dataset number dset (defaults to 1) by numCounts counts.\n" +
    "This routine uses the 'grpNumCounts' function from the group library.\n" +
    usage_filter_str;

  variable dset, numCounts;
  switch ( _NARGS )
  { case 2: ( dset, numCounts ) = (); }
  { case 1: dset = 1; numCounts = (); }
  { usage( usage_str ); }
  doGrouping( dset, &grpNumCounts, numCounts );
}

%
% Usage:
%   groupBySNR( dset, minSNR );
%
% Aim:
%   groups dataset # dset so that the minimum signal-to-noise
%   ratio is at least minSNR
%
public define groupBySNR() {
  variable usage_str =
    _function_name + "( [dset], minSNR );\n\n" +
    "Groups dataset number dset (defaults to 1) using a signal-to-noise ratio\n" +
    "of at least minSNR.\n" +
    "This routine uses the 'grpSnr' function from the group library.\n" +
    usage_filter_str;

  variable dset, minSNR;
  switch ( _NARGS )
  { case 2: ( dset, minSNR ) = (); }
  { case 1: dset = 1; minSNR = (); }
  { usage( usage_str ); }
  doGrouping( dset, &grpSnr, minSNR );
}

%
% Usage:
%   groupAdaptively( dset, minCounts );
%
% Aim:
%   adaptive grouping scheme to achieve at least minCounts per
%   bin. It is slightly different to groupByCounts() since it
%   does not work "left to right" - i.e. march through the
%   array and group bins until the counts criterion is exceeded -
%   instead it works by numbed of counts. See the
%   description of grpAdaptive() for more details.
%
% Note:
%   This algorithm is designed to enhance the significance of
%   emission rather than absorption features. 
%
public define groupAdaptively() {
  variable usage_str =
    _function_name + "( [dset], minCounts );\n\n" +
    "Adaptively groups dataset number dset (defaults to 1) so that bins have\n" +
    "at least minCounts counts.\n" +
    "This routine uses the 'grpAdaptive' function from the group library.\n" +
    usage_filter_str;

  variable dset, minCounts;
  switch ( _NARGS )
  { case 2: ( dset, minCounts ) = (); }
  { case 1: dset = 1; minCounts = (); }
  { usage( usage_str ); }
  doGrouping( dset, &grpAdaptive, minCounts );
}

%
% Usage:
%   groupAdaptiveSNR( dset, minSNR );
%
% Aim:
%   adaptive grouping scheme to achieve a signal-to-noise
%   of at least minSNR per bin.
%   It is slightly different to groupBySNR() since it
%   does not work "left to right" - i.e. march through the
%   array and group bins until the counts criterion is exceeded -
%   instead it works by numbed of counts. See the
%   description of grpAdaptive() for more details.
%
% Note:
%   This algorithm is designed to enhance the significance of
%   emission rather than absorption features. 
%
public define groupAdaptiveSNR() {
  variable usage_str =
    _function_name + "( [dset], minSNR );\n\n" +
    "Adaptively groups dataset number dset (defaults to 1) using a signal-to-noise ratio\n" +
    "of at least minSNR.\n" +
    "This routine uses the 'grpAdaptiveSnr' function from the group library.\n" +
    usage_filter_str;

  variable dset, minSNR;
  switch ( _NARGS )
  { case 2: ( dset, minSNR ) = (); }
  { case 1: dset = 1; minSNR = (); }
  { usage( usage_str ); }
  doGrouping( dset, &grpAdaptiveSnr, minSNR );
}

provide( "sherpa_utils" );

%%%
%%% End
%%%
