%
% $Id: show_tgain_corr.sl,v 1.7 2007/09/13 10:44:01 egalle Exp $
%
% Aim:
%  Routines for showing how the time-dependent gain correction
%  changes the gain.
%
% Notes:
%  - algorithm/code taken from Glenn Alenn
%  - uses ISIS for file input, plotting, and data manipulation
%    Varmm and ChIPS could have been used instead, except that
%    we would have then had to write our own interpolation routine
%  - relies on the files being in the CALDB but hard codes their
%    names. Should we query the CALDB for the files?
%

private variable caldb = getenv("CALDB");
if ( NULL == caldb )
  verror( "Unable to find CALDB env. variable" );
caldb += "/data/chandra/acis/bcf/";

% perhaps we should allow users to change these files
%
private variable gain_file = 
  "acisD2000-01-29gain_ctiN0006.fits";

private variable tgain_files = 
  [
    "acisD2002-08-01t_gainN0006.fits",
    "acisD2007-02-01t_gain_biN0002.fits",
    "acisD2007-02-01t_gainN0006.fits",
    "acisD2007-05-01t_gain_biN0002.fits"
  ];

private variable plots = Assoc_Type [Struct_Type];
plots["energy"] =
  struct { xrange, xline, xlabel, ylabel, xpos, xlog, yrange };
set_struct_fields( plots["energy"], [0.1,10], [0.13,0.2],
		   "Energy (keV)", "Change in energy (eV)",
		   0.22, 1,
		   _reshape( [-20,150, -20,150, -20,150, -20,150,
			      -20,150, -5,40, -20,150, -5,40, -20,150,-20,150], [10,2] )
		   );

if ( _isis_version < 10107 )
  plots["energy"].xpos = log10( plots["energy"].xpos );

plots["pha"] =
  struct { xrange, xline, xlabel, ylabel, xpos, xlog, yrange };
set_struct_fields( plots["pha"], [0,4100], [200,500],
		   "PHA (adu)", "Change in PHA (adu)",
		   600, 0,
		   _reshape( [-10,50, -10,50, -10,50, -10,50,
			      -10,50, -5,15, -10,50, -5,15, -10,50,-10,50], [10,2] )
		   );

%
% Usage:
%   show_tgain_corr( outfile, outdev, type, location, title, verbose );
%
% Aim:
%   create a plot that shows the time-dependant gain
%   correction. Outfile is the name of the file containing the 
%   plot (which is created using the outdev device). Common values
%   of outdev will be "cps" and "gif".
%
%   type is "pha" for a plot in PHA or "energy" for a plot in ev vs keV
%
%   location is a struct containing the fields: ccd_id, chipx, chipy
%
% Notes:
%   
%
define show_tgain_corr (outfile,outdev,type,loc,title_str,verbose) {

  !if( assoc_key_exists( plots, type ) )
    verror( "Unknown plot type of '%s'. Must be one of: %s\n",
	    type, strjoin( assoc_get_keys(plots), " " ) );

  variable ymin = plots[type].yrange[loc.ccd_id,0];
  variable ymax = plots[type].yrange[loc.ccd_id,1];

  variable infile = caldb + "gain/" + gain_file;
  if ( 3 < verbose )
    vmessage( "Gain file = %s", infile );
  variable gain = fits_read_table( infile );
%%  if ( NULL == gain )
%%    verror( "Unable to read in gain file: %s", infile );

  %
  % find the rows in the gain file for this chip location
  %
  variable idx_gain = where (
			     loc.ccd_id == gain.ccd_id and
			     loc.chipx >= gain.chipx_min and
			     loc.chipx <= gain.chipx_max and
			     loc.chipy >= gain.chipy_min and
			     loc.chipy <= gain.chipy_max
			     );
  if ( 0 == length(idx_gain) )
    verror( "Unable to find any gain data for chip=%d %d on CCD %d\n",
	    loc.chipx, loc.chipy, loc.ccd_id );
  idx_gain = idx_gain[0];

  variable npts    = gain.npoints[idx_gain];
  variable gpha    = gain.pha[idx_gain,[[0:npts-1]]];
  variable genergy = gain.energy[idx_gain,[[0:npts-1]]];
  if ( 5 == verbose )
    vmessage( "Found %d pha/energy points in the gain file", npts );

  %
  % set up plot
  %
  variable fp = open_plot( outfile + "/" + outdev );
%%  if ( NULL == fp )
%%    verror( "Unable to create %s file to %s\n", outdev, outfile );

  color( 1 );
  connect_points( 1 );
  linestyle( 1 );
  set_frame_line_width( 2 );
  set_line_width( 2 );
  if ( "" == title_str )
    title( sprintf( "ccd=%d x=%d y=%d", loc.ccd_id, loc.chipx, loc.chipy ) );
  else
    title( sprintf( "%s [ccd=%d x=%d y=%d]", title_str, loc.ccd_id, loc.chipx, loc.chipy ) );

  xlabel( plots[type].xlabel );
  if ( plots[type].xlog )
    xlog;
  else
    xlin;
  xrange( plots[type].xrange[0], plots[type].xrange[1] );
  if ( 3 < verbose )
    vmessage( "Setting x range of plot to %g to %g logged=%d",
	      plots[type].xrange[0], plots[type].xrange[1], plots[type].xlog );

  ylabel( plots[type].ylabel );
  yrange( ymin, ymax );
  if ( 3 < verbose )
    vmessage( "Setting y range of plot to %g to %g", ymin, ymax );

  plot( plots[type].xrange, [0, 0] );

  color( 1 );
  connect_points( 1 );
  linestyle( 1 );
  set_line_width( 2 );

  variable dy = ( ymax - ymin ) / 20.0;
  variable y = ymax - 1.5 * dy;
  variable xline = plots[type].xline;
  variable x = plots[type].xpos;

  if ( 3 < verbose )
    vmessage( "Labels start at y=%g with delta_y=%g for x= %g to %g with text starting at %g",
	      y, dy, xline[0], xline[1], x );

  oplot( xline, [y, y] );

  color( 1 );
  xylabel( x, y, "No change (2000-01-29)" );
  
  % loop through the tgain files
  %
  _for (0, length(tgain_files)-1, 1) {
    variable i = ();
  
    infile = caldb + "t_gain/" + tgain_files[i];
    if ( 3 < verbose )
      vmessage( "Time-dependant gain file = %s", infile );
    variable tgain = fits_read_table( infile );
    if ( NULL == tgain )
      verror( "Unable to open time-dependant gain file %s\n", infile );

    variable idx_tgain = where (
				loc.ccd_id == tgain.ccd_id and
				loc.chipx >= tgain.chipx_lo and
				loc.chipx <= tgain.chipx_hi and
				loc.chipy >= tgain.chipy_lo and
				loc.chipy <= tgain.chipy_hi
				);
    if ( 0 == length(idx_tgain) )
      verror( "Unable to find any time-dependant gain data for chip=%d %d on CCD %d\n",
	      loc.chipx, loc.chipy, loc.ccd_id );
    idx_tgain = idx_tgain[0];
    if ( 5 == verbose )
      vmessage( "Found %d points in the time-dependant gain file", tgain.npoints[idx_tgain] );

    variable xpts, ypts;
    switch ( type )
    {
    case "energy":
      variable energy = interpol( tgain.pha[idx_tgain,*], gpha, genergy );
      variable energy_adj = interpol( tgain.pha[idx_tgain,*]-(tgain.deltpha2[idx_tgain,*]+tgain.deltpha1[idx_tgain,*])/2.0,gpha, genergy );
      xpts = energy / 1.0e3;
      ypts = energy_adj - energy;
    }
    {
    case "pha":
      xpts = tgain.pha[idx_tgain,*];
      ypts = - (tgain.deltpha1[idx_tgain,*]+tgain.deltpha2[idx_tgain,*])/2.0;
    }

    color( i+2 );
    connect_points( 1 );
    linestyle( 1 );
    set_line_width( 2 );
    oplot( xpts, ypts );
      
    color( i+2 );
    connect_points( 1 );
    linestyle( 1 );
    set_line_width( 2 );
    y -= dy;
    oplot( xline, [y, y] );

    color( 1 );
    xylabel( x, y, tgain_files[i] );

  } % _for: 0 .. length(tgain_files)-1

  close_plot( fp );

} % plot_tgain_corr()

provide( "show_tgain_corr" );

