%
% $Id: sstats.sl,v 0.4 2001/10/04 18:45:12 dburke Exp $
%
% Usage:
%   sstats( filename )
%   sstats( filename, maskname )
%
% Example:
%     chips> () = evalfile( "sstats.sl" );
%     chips> sstats( "img.fits" );
%     chips> sstats( "img.fits", "emap.fits" );
%
%   or
%
%     unix% chips --slscript sstats.sl
%     chips> sstats( "img.fits" );
%     chips> sstats( "img.fits", "emap.fits" );
%
% Thread:
%   http://cxc.harvard.edu/ciao/threads/image_stats/
%
% Aim:
%   calculate simple statistics of images 
%   (accounting for masked regions if desired)
%
%   The mean, RMS, min, max and total values are calculated
%   and printed to the screen. Example 6 of "ahelp slang-tips"
%   shows how the routine can be extended to calculate the
%   median value of an array
%
% Notes:
%   Will work in both ChIPS and Sherpa (with CIAO 2.1 it would
%   only work in ChIPS)
%   

% outupt routines - thanks to John Davis
%
define output (x) {
    () = fputs (x, stdout);
    () = fputs ("\n", stdout);
}
    
define voutput () {
    variable args = __pop_args (_NARGS);
    output (sprintf (__push_args (args)));
}

% Usage:
%    stats = calc_stats( array_var )
%
%   where stats is a structure with the
%   following fields:
%     mean, rms, min, max, total, ngood
%
% Aim:
%  calculate simple stats for the supplied array
%  array_var can be of any dimensionality - all
%  elements are used in the calculation. If the
%  array is > 1D, then a 1D copy of the array is 
%  made, which means the routine can use a lot
%  of memory if sent in big arrays
%
define calc_stats() {

    if (_NARGS != 1) {
        usage( "() = calc_stats(a)\nwhere a is an array of numbers" );
    }

    variable a = ();
    if (typeof(a) != Array_Type) {
        usage( "() = calc_stats(a)\nwhere a is an array of numbers" );
    }  

    variable npts = length(a);

    % Find out the size of the array: reshape if not 1D
    variable n1;
    ( , n1, ) = array_info( a );
    if ( n1 > 1 ) {
        a = _reshape( a, [npts] ); % make a copy so that do not change input array
    }

    % calculate the stats
    variable total, minval, maxval, value;
   
    total  = 0;
    minval = a[0];
    maxval = a[0];

    % loop through remainder of array
    foreach (a) {
	value = ();
        total += value;
        if ( value < minval )      { minval = value; }
        else if ( value > maxval ) { maxval = value; }
    }

    % ensure we have a floating-point calculation
    variable mean = total * 1.0 / npts;

    % calculate the RMS using the corrected two-pass algorithm
    variable diff, sum, sumsq;
    sum   = 0.0;
    sumsq = 0.0;
    foreach (a) {
        diff   = () - mean;
        sum   += diff;
        sumsq += (diff*diff);
    }

    % again ensure we're all floating point
    variable rms = sqrt( (sumsq - sum*sum * 1.0 / npts ) / (npts-1.0) );

    variable stats = struct { mean, rms, minval, maxval, total, npts };
    set_struct_fields( stats, mean, rms, minval, maxval, total, npts );
    return stats;

} % calc_stats()

% Usage:
%   check_sizes( img1, img2 )
%
% Aim:
%   exits if the two images are not the same size
%
% Note:
%   MUST be sent the image structure, not the pixels element
%
define check_sizes() {

    if (_NARGS != 2) {
        usage( "check_sizes(a,b)\nwhere a and b are image structures" );
    }

    variable a, b;
    ( a, b ) = ();

    if ( a._naxes != b._naxes ) {
        error( "The two images must have the same dimensionality." );
    }

    variable d1, n1, t1, d2, n2, t2, i;
    ( d1, n1, t1 ) = array_info( a.pixels );
    ( d2, n2, t2 ) = array_info( b.pixels );
    for ( i = 0; i < n1; i++ ) {
        if ( d1[i] != d2[i] ) {
            error( "The two images must be the same size (axis " + string(i) + ")." );
        }
    }

} % check_sizes()

% Usage:
%   sstats( filename )
%   sstats( filename, maskname )
%
% Aim:
%   Calculate simple statistics for an image.
%   If a second image is supplied, it is taken to be a mask,
%   so that only those pixels whose values are > 0 in the
%   mask are included in the calculation.
%   (for instance, use an exposure map which has been set to
%   0 for regions you do not want included in the calculation)
%
define sstats() {
  
    % read in images
    variable filename, maskname, image, mask;
    if ( _NARGS == 2 ) {
        ( filename, maskname ) = ();
        image = readimage( filename );
        if ( image == NULL ) {
	    error( "sstats() has stopped: unable to read image file." );
        }
        mask = readimage( maskname );
        if ( mask == NULL ) {
	    error( "sstats() has stopped: unable to read mask file." );
        }

    } else if ( _NARGS == 1 ) {
        filename = ();
        image = readimage( filename );
        if ( image == NULL ) {
	    error( "sstats() has stopped: unable to read image file." );
        }
	mask = NULL;
    } else {
        usage( "sstats(filename[,maskname])\nwhere the files are FITS images." );
    }

    % returned statistics
    variable stats;

    % safety checks
    if ( mask != NULL ) {
        check_sizes( image, mask );

        % remove "unwanted" points
        variable index = where( mask.pixels > 0 );
        if ( length(index) == 0 ) {
            error( "maskname does not contain any pixels > 0." );
        }

        voutput( "Statistics of %s using (%s > 0) as a mask.", filename, maskname );
        stats = calc_stats( image.pixels[index] );

    } else {
        voutput( "Statistics of %s.", filename );
        stats = calc_stats( image.pixels );
    }

    voutput( "Mean = %11.4e  RMS = %11.4e  Total      = %11.4e", 
	stats.mean, stats.rms, stats.total ); 
    voutput( "Min  = %11.4e  Max = %11.4e  Num pixels = %d", 
	stats.minval, stats.maxval, stats.npts );

    return;

} % sstats()

%% end of S-Lang code
